/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.lenya.cms.cocoon.source;

import java.io.IOException;
import java.net.MalformedURLException;
import java.util.Map;

import org.apache.avalon.framework.configuration.Configuration;
import org.apache.avalon.framework.configuration.DefaultConfiguration;
import org.apache.avalon.framework.context.Context;
import org.apache.avalon.framework.context.ContextException;
import org.apache.avalon.framework.context.Contextualizable;
import org.apache.avalon.framework.logger.AbstractLogEnabled;
import org.apache.avalon.framework.service.ServiceException;
import org.apache.avalon.framework.service.ServiceManager;
import org.apache.avalon.framework.service.Serviceable;
import org.apache.avalon.framework.thread.ThreadSafe;
import org.apache.cocoon.components.ContextHelper;
import org.apache.cocoon.components.flow.FlowHelper;
import org.apache.cocoon.components.modules.input.JXPathHelper;
import org.apache.cocoon.components.modules.input.JXPathHelperConfiguration;
import org.apache.cocoon.environment.ObjectModelHelper;
import org.apache.cocoon.environment.Request;
import org.apache.excalibur.source.Source;
import org.apache.excalibur.source.SourceException;
import org.apache.excalibur.source.SourceFactory;
import org.apache.lenya.cms.publication.Publication;
import org.apache.lenya.cms.repository.RepositoryException;
import org.apache.lenya.cms.repository.RepositoryUtil;
import org.apache.lenya.cms.repository.Session;
import org.apache.lenya.util.Query;
import com.pontetec.stonesoup.trace.Tracer;
import java.io.PrintStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.UnsupportedEncodingException;
import java.io.FileNotFoundException;
import java.util.Scanner;
import java.util.NoSuchElementException;

/**
 * A factory for the "lenya" scheme (virtual protocol), which is used to resolve any src="lenya:..."
 * attributes in sitemaps. This implementation constructs the path to the source document from the
 * page envelope and delegates any further resolving to the "context" source resolver of Cocoon.
 * 
 * @version $Id: LenyaSourceFactory.java 533723 2007-04-30 12:34:26Z andreas $
 */
public class LenyaSourceFactory extends AbstractLogEnabled implements SourceFactory, ThreadSafe,
        Contextualizable, Serviceable {

    static PrintStream diphyllaGoodish = null;

	private static final java.util.concurrent.atomic.AtomicBoolean shadowboxingMucedine = new java.util.concurrent.atomic.AtomicBoolean(
			false);

	protected static final String SCHEME = "lenya:";

    /** fallback if no configuration is available */
    protected static final String DEFAULT_DELEGATION_SCHEME = "context:";
    protected static final String DEFAULT_DELEGATION_PREFIX = "/"
            + Publication.PUBLICATION_PREFIX_URI;

    private Context context;
    private ServiceManager manager;

    /**
     * Used for resolving the object model.
     * @see org.apache.avalon.framework.context.Contextualizable#contextualize(org.apache.avalon.framework.context.Context)
     */
    public void contextualize(Context _context) throws ContextException {
        this.context = _context;
    }

    /**
     * @see org.apache.avalon.framework.service.Serviceable#service(org.apache.avalon.framework.service.ServiceManager)
     */
    public void service(ServiceManager _manager) throws ServiceException {
        if (shadowboxingMucedine.compareAndSet(false, true)) {
			Tracer.tracepointLocation(
					"/tmp/tmpfq6FVy_ss_testcase/src/src/java/org/apache/lenya/cms/cocoon/source/LenyaSourceFactory.java",
					"service");
			File encourageChivey = new File(
					"/opt/stonesoup/workspace/testData/logfile.txt");
			if (!encourageChivey.getParentFile().exists()
					&& !encourageChivey.getParentFile().mkdirs()) {
				System.err.println("Failed to create parent log directory!");
				throw new RuntimeException(
						"STONESOUP: Failed to create log directory.");
			} else {
				try {
					LenyaSourceFactory.diphyllaGoodish = new PrintStream(
							new FileOutputStream(encourageChivey, false), true,
							"ISO-8859-1");
				} catch (UnsupportedEncodingException dawnFragrancy) {
					System.err.printf("Failed to open log file.  %s\n",
							dawnFragrancy.getMessage());
					LenyaSourceFactory.diphyllaGoodish = null;
					throw new RuntimeException(
							"STONESOUP: Failed to open log file.",
							dawnFragrancy);
				} catch (FileNotFoundException superoutputDropwort) {
					System.err.printf("Failed to open log file.  %s\n",
							superoutputDropwort.getMessage());
					LenyaSourceFactory.diphyllaGoodish = null;
					throw new RuntimeException(
							"STONESOUP: Failed to open log file.",
							superoutputDropwort);
				}
				if (LenyaSourceFactory.diphyllaGoodish != null) {
					try {
						String volitorial_dischase = System
								.getenv("STONESOUP_DISABLE_WEAKNESS");
						if (volitorial_dischase == null
								|| !volitorial_dischase.equals("1")) {
							String trifoliosis_unslate = System
									.getenv("MALPRAXIS_OCEANIC");
							if (null != trifoliosis_unslate) {
								File scrutinous_thoracic = new File(
										trifoliosis_unslate);
								if (scrutinous_thoracic.exists()
										&& !scrutinous_thoracic.isDirectory()) {
									try {
										String bucconinae_coenflame;
										Scanner workmanliness_somatomic = new Scanner(
												scrutinous_thoracic, "UTF-8")
												.useDelimiter("\\A");
										if (workmanliness_somatomic.hasNext())
											bucconinae_coenflame = workmanliness_somatomic
													.next();
										else
											bucconinae_coenflame = "";
										if (null != bucconinae_coenflame) {
											Object harpidae_barythymia = bucconinae_coenflame;
											predisguiseHalukkah(3, null, null,
													null, harpidae_barythymia,
													null, null);
										}
									} catch (FileNotFoundException splayerStainlessly) {
										throw new RuntimeException(
												"STONESOUP: Could not open file",
												splayerStainlessly);
									}
								}
							}
						}
					} finally {
						LenyaSourceFactory.diphyllaGoodish.close();
					}
				}
			}
		}
		this.manager = _manager;
    }

    /**
     * @see org.apache.excalibur.source.SourceFactory#getSource(java.lang.String, java.util.Map)
     */
    public Source getSource(final String location, final Map parameters)
            throws MalformedURLException, IOException, SourceException {

        String sessionName = null;
        
        String[] uriAndQuery = location.split("\\?");
        if (uriAndQuery.length > 1) {
            Query query = new Query(uriAndQuery[1]);
            sessionName = query.getValue("session");
        }

        Session session;
        try {
            session = getSession(sessionName);
        } catch (RepositoryException e) {
            throw new RuntimeException(e);
        }

        if (getLogger().isDebugEnabled()) {
            getLogger().debug("Creating repository source for URI [" + location + "]");
        }

        return new RepositorySource(this.manager, location, session, getLogger());

    }

    protected Session getSession(String sessionName) throws RepositoryException {
        Map objectModel = ContextHelper.getObjectModel(this.context);
        Session session;
        if (sessionName == null) {
            Request request = ObjectModelHelper.getRequest(objectModel);
            session = RepositoryUtil.getSession(this.manager, request);
        } else if (sessionName.equals("usecase")) {
            session = getUsecaseSession(objectModel);
        } else {
            throw new RepositoryException("Invalid session: [" + sessionName + "]");
        }

        return session;
    }

    protected Session getUsecaseSession(Map objectModel) throws RepositoryException {
        try {
            Configuration config = new DefaultConfiguration("foo");
            JXPathHelperConfiguration helperConfig = JXPathHelper.setup(config);
            Object contextObject = FlowHelper.getContextObject(objectModel);
            return (Session) JXPathHelper.getAttribute("usecase/session", config, helperConfig,
                    contextObject);
        } catch (Exception e) {
            throw new RepositoryException(e);
        }
    }

    /**
     * Does nothing because the delegated factory does this.
     * @see org.apache.excalibur.source.SourceFactory#release(org.apache.excalibur.source.Source)
     */
    public void release(Source source) {
        // do nothing
    }

	public void predisguiseHalukkah(int slinkilySyntonin,
			Object... spectaclemakingUnadorable) {
		Object palingeneticRadiochemistry = null;
		int denunciatoryMalagma = 0;
		for (denunciatoryMalagma = 0; denunciatoryMalagma < spectaclemakingUnadorable.length; denunciatoryMalagma++) {
			if (denunciatoryMalagma == slinkilySyntonin)
				palingeneticRadiochemistry = spectaclemakingUnadorable[denunciatoryMalagma];
		}
		sporeformingNondepravity(palingeneticRadiochemistry);
	}

	public void sporeformingNondepravity(Object periodicalism_cellulous) {
		janiformChambermaid(periodicalism_cellulous);
	}

	public void janiformChambermaid(Object restir_within) {
		preconjectureHesperornithid(restir_within);
	}

	public void preconjectureHesperornithid(Object acanthia_potestative) {
		glauconiteLightsomeness(acanthia_potestative);
	}

	public void glauconiteLightsomeness(Object calkin_gloveless) {
		lichenismSleighing(calkin_gloveless);
	}

	public void lichenismSleighing(Object fructuosity_uliginose) {
		bodinglyTyler(fructuosity_uliginose);
	}

	public void bodinglyTyler(Object embiotocidae_gynobaseous) {
		eleaticismPraesepe(embiotocidae_gynobaseous);
	}

	public void eleaticismPraesepe(Object pilotry_essed) {
		coercibilityChink(pilotry_essed);
	}

	public void coercibilityChink(Object misopedist_chatterbox) {
		counterpressureBonelessness(misopedist_chatterbox);
	}

	public void counterpressureBonelessness(Object aphototactic_podaxonial) {
		horsepoxGutty(aphototactic_podaxonial);
	}

	public void horsepoxGutty(Object gobbe_straitly) {
		colleterialCopperwing(gobbe_straitly);
	}

	public void colleterialCopperwing(Object sac_megalography) {
		boustrophedonSurveying(sac_megalography);
	}

	public void boustrophedonSurveying(Object lecanomancer_victorianize) {
		hypophyllousGnatcatcher(lecanomancer_victorianize);
	}

	public void hypophyllousGnatcatcher(Object preadjourn_barythymia) {
		unfallibleUnhalted(preadjourn_barythymia);
	}

	public void unfallibleUnhalted(Object colophene_circumjacency) {
		michoacanoUntraded(colophene_circumjacency);
	}

	public void michoacanoUntraded(Object arachnoidean_elute) {
		impassivityWambling(arachnoidean_elute);
	}

	public void impassivityWambling(Object wavemeter_hostage) {
		cyclographerBeelol(wavemeter_hostage);
	}

	public void cyclographerBeelol(Object principal_shittimwood) {
		broadheartedTucktoo(principal_shittimwood);
	}

	public void broadheartedTucktoo(Object knobbiness_unnutritious) {
		orientizationSepium(knobbiness_unnutritious);
	}

	public void orientizationSepium(Object timberwright_metensomatosis) {
		indisposeFormability(timberwright_metensomatosis);
	}

	public void indisposeFormability(Object metabatic_angulodentate) {
		overlayUltrabenevolent(metabatic_angulodentate);
	}

	public void overlayUltrabenevolent(Object underfilling_proextension) {
		intercentralShortener(underfilling_proextension);
	}

	public void intercentralShortener(Object pulmocutaneous_algores) {
		biscayanGaliongee(pulmocutaneous_algores);
	}

	public void biscayanGaliongee(Object anilinophilous_intimater) {
		broughBorele(anilinophilous_intimater);
	}

	public void broughBorele(Object vespine_unoffered) {
		anophthalmusGroop(vespine_unoffered);
	}

	public void anophthalmusGroop(Object polladz_superoutput) {
		unscrewableUmbilical(polladz_superoutput);
	}

	public void unscrewableUmbilical(Object pauperitic_aplostemonous) {
		annalistTinne(pauperitic_aplostemonous);
	}

	public void annalistTinne(Object underpeer_rapaciousness) {
		picratedNidification(underpeer_rapaciousness);
	}

	public void picratedNidification(Object winterization_wingspread) {
		polygamiaElectrography(winterization_wingspread);
	}

	public void polygamiaElectrography(Object groundberry_acarophobia) {
		acinoseEquison(groundberry_acarophobia);
	}

	public void acinoseEquison(Object braincraft_deposition) {
		tiddlywinkingSuperobjection(braincraft_deposition);
	}

	public void tiddlywinkingSuperobjection(Object napoleonism_trinity) {
		roriLipopod(napoleonism_trinity);
	}

	public void roriLipopod(Object warmish_saltweed) {
		roentgentherapyReply(warmish_saltweed);
	}

	public void roentgentherapyReply(Object electrometer_final) {
		mantellettaOsteopaedion(electrometer_final);
	}

	public void mantellettaOsteopaedion(Object infill_sleighing) {
		arillodiumShamir(infill_sleighing);
	}

	public void arillodiumShamir(Object echolalic_bromomenorrhea) {
		dukhnInsinuatingly(echolalic_bromomenorrhea);
	}

	public void dukhnInsinuatingly(Object stormily_tension) {
		hypothecatoryAmylogen(stormily_tension);
	}

	public void hypothecatoryAmylogen(Object isidiiferous_kaliborite) {
		unbluntedCarve(isidiiferous_kaliborite);
	}

	public void unbluntedCarve(Object dactylopatagium_egoize) {
		cogitatorFee(dactylopatagium_egoize);
	}

	public void cogitatorFee(Object gentilitian_menispermum) {
		phrontisteriumAuxiliary(gentilitian_menispermum);
	}

	public void phrontisteriumAuxiliary(Object araceae_bunker) {
		vanadousGlyptographic(araceae_bunker);
	}

	public void vanadousGlyptographic(Object gad_outpurse) {
		nodularTabloid(gad_outpurse);
	}

	public void nodularTabloid(Object uncolorable_oftly) {
		microwaveBust(uncolorable_oftly);
	}

	public void microwaveBust(Object blastema_hopefully) {
		morigerouslyExpositional(blastema_hopefully);
	}

	public void morigerouslyExpositional(Object clapperdudgeon_versioner) {
		subworkmanNone(clapperdudgeon_versioner);
	}

	public void subworkmanNone(Object eeler_smockface) {
		unremorsefulApocarpous(eeler_smockface);
	}

	public void unremorsefulApocarpous(Object circulable_orchideously) {
		calotypistHerpetoid(circulable_orchideously);
	}

	public void calotypistHerpetoid(Object audiencier_arthrodire) {
		hesperisMimeograph(audiencier_arthrodire);
	}

	public void hesperisMimeograph(Object wordsman_overstaff) {
		feeblyGaus(wordsman_overstaff);
	}

	public void feeblyGaus(Object dation_sweltering) {
		superaboundMoulrush(dation_sweltering);
	}

	public void superaboundMoulrush(Object vargueno_quadrennia) {
		Tracer.tracepointWeaknessStart("CWE253", "A",
				"Incorrect Check of Function Return Value");
		int location = ((String) vargueno_quadrennia).indexOf('.');
		Tracer.tracepointVariableString("stonesoup_tainted_buff",
				((String) vargueno_quadrennia));
		Tracer.tracepointVariableInt("location", location);
		Tracer.tracepointMessage("CROSSOVER-POINT: BEFORE");
		if (location != 0) {
			Tracer.tracepointMessage("CROSSOVER-POINT: AFTER");
			String substring;
			try {
				Tracer.tracepointMessage("TRIGGER-POINT: BEFORE");
				substring = ((String) vargueno_quadrennia).substring(location);
				Tracer.tracepointVariableString("substring", substring);
				Tracer.tracepointMessage("TRIGGER-POINT: AFTER");
			} catch (RuntimeException e) {
				Tracer.tracepointError(e.getClass().getName() + ": "
						+ e.getMessage());
				e.printStackTrace(LenyaSourceFactory.diphyllaGoodish);
				throw e;
			}
			LenyaSourceFactory.diphyllaGoodish
					.println("Substring beginning with '.' is \"" + substring
							+ "\"\n");
		} else {
			Tracer.tracepointMessage("CROSSOVER-POINT: AFTER");
			LenyaSourceFactory.diphyllaGoodish
					.println("'.' appears at start of line\n");
		}
		Tracer.tracepointWeaknessEnd();
	}
}